package net.sf.sido.factory.sido.antlr;

import net.sf.sido.DataObject;
import net.sf.sido.factory.support.SiDOSchema;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.Validate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class SidoReporter implements ISidoReporter {

	private static final Logger log = LoggerFactory.getLogger(SidoReporter.class);

	private final SiDOSchema sido;

	private DataObject schema;
    private DataObject type;
    private DataObject property;
    private DataObject constraint;

	public SidoReporter(SiDOSchema sidoSchema) {
        this.sido = sidoSchema;
	}

	public DataObject getSchema() {
		if (schema == null) {
			throw new IllegalStateException("No schema has been parsed.");
		} else {
			return schema;
		}
	}

    @Override
    public void closeType() {
        log.debug("close type");
        Validate.notNull(type, "No type is defined.");
        type = null;
    }

    @Override
	public void setAbstract() {
		log.debug("abstract");
        Validate.notNull(type, "No type is defined.");
        type.setBoolean(SiDOSchema.ABSTRACT, true);
	}

	@Override
	public void setAnonymousProperty() {
		log.debug("anonymous");
        Validate.notNull(property, "No property is defined.");
        property.setBoolean(SiDOSchema.ANONYMOUS, true);
	}

	@Override
	public void createConstraint(String id) {
		log.debug("constraint " + id);
        Validate.notNull(property, "No property is defined.");
        constraint = sido.createConstraint(property, id);
	}

	@Override
	public void setConstraintParam(String name, String value) {
		log.debug("constraint param " + name + " = " + value);
        Validate.notNull(constraint, "No constraint is defined.");
        sido.setConstraintParam(constraint, name, value);
	}

	@Override
	public void setConstraintValue(String value) {
		log.debug("constraint value " + value);
        Validate.notNull(constraint, "No constraint is defined.");
        constraint.setString(SiDOSchema.VALUE, value);
	}

	@Override
	public void setExtensible() {
		log.debug("extensible");
        Validate.notNull(type, "No type is defined.");
        type.setBoolean(SiDOSchema.EXTENSIBLE, true);
	}

	@Override
	public void createExtension(String forType) {
		log.debug("extension for " + forType);
        Validate.notNull(schema, "The schema has not been defined yet.");
        Validate.isTrue(type == null, "A type is currently built.");
        type = sido.createExtension(schema, forType);
	}

	@Override
	public void createProperty() {
		log.debug("property new");
        Validate.notNull(type, "No type is defined.");
        Validate.isTrue(property == null, "A property is currently built.");
        property = sido.createProperty(type);
	}

	@Override
	public void setParent(String parentTypeName) {
		log.debug("parent " + parentTypeName);
        Validate.notNull(type, "No type is defined.");
        type.setString(SiDOSchema.PARENT, parentTypeName);
	}

	@Override
	public void setPropertyName(String name) {
		name = StringUtils.stripStart(name, "#");
		log.debug("property " + name);
        Validate.notNull(property, "No property is defined.");
        property.setString(SiDOSchema.NAME, name);
        property = null;
	}

	@Override
	public void setPropertyArray() {
		log.debug("array ");
        Validate.notNull(property, "No property is defined.");
        property.setBoolean(SiDOSchema.ARRAY, true);
	}
	
	@Override
	public void setPropertyArrayIndex(String indexPropertyPath) {
		log.debug("array index " + indexPropertyPath);
		Validate.isTrue (StringUtils.isNotBlank (indexPropertyPath), "The index property path must not be blank.");
		property.setString(SiDOSchema.ARRAY_INDEX, indexPropertyPath);
	}

	@Override
	public void setPropertyNullable(boolean not) {
		log.debug("nullable " + not);
        Validate.notNull(property, "No property is defined.");
        property.setBoolean(SiDOSchema.NULLABLE, not);
	}

	@Override
	public void setPropertyReverse() {
		log.debug("reverse");
        Validate.notNull(property, "No property is defined.");
        property.setBoolean(SiDOSchema.REVERSE, true);
	}

	@Override
	public void setPropertyType(String typeName) {
		log.debug("type " + typeName);
        Validate.notNull(property, "No property is defined.");
        property.setString(SiDOSchema.TYPEREF, typeName);
	}

	@Override
	public void createSchema(String name) {
		log.debug("schema " + name);
        Validate.isTrue(schema == null, "Schema has already been declared");
		schema = sido.createSchema(name);
	}

	@Override
	public void createType(String name) {
		log.debug("type " + name);
        Validate.notNull(schema, "The schema has not been defined yet.");
        Validate.isTrue(type == null, "A type is currently built.");
        type = sido.createType(schema, name);
	}

}
